-- Fischer's mutual exclusion protocol

define(c1,1)	
define(d1,2)	
define(c2,1)	
define(d2,2)	
define(c3,1)	
define(d3,2)	
define(c4,1)	
define(d4,2)	
define(a,1)		-- max delay time to register intent
define(b,4)	-- a*d2/10+1

var 
	x1,
	x2,
	x3,
	x4,
	k
	: analog;
	
 -- -------------------------------------------------------------- *)

automaton proc1
synclabs : tau,start1,set1,enter1,abort1,release1;

initially idle & True;

loc idle: while True wait {dx1 in [c1,d1]}
	when k=0 sync start1 do {x1' =0} goto start;
	when True sync tau do {x1'=x1 , k'=k'} goto idle;
	
loc start: while x1<=a wait {dx1 in [c1,d1]}
	when True sync set1 do {k' =1 , x1' =0} goto check;
	when True sync tau do {x1'=x1 , k'=k'} goto start;

loc check: while True wait {dx1 in [c1,d1]}
	when k=1 & x1>=b sync enter1 goto CS;
	when k<1 & x1>=b sync abort1 goto idle;
	when k>1 & x1>=b sync abort1 goto idle;
	when True sync tau do {x1'=x1 , k'=k'} goto check;

loc CS: while True wait {dx1 in [c1,d1]}
	when True sync release1 do {k' = 0} goto idle;
	when True sync tau do {x1'=x1 , k'=k'} goto CS;
end

automaton proc2
synclabs : tau,start2,set2,enter2,abort2,release2;

initially idle & True;

loc idle: while True wait {dx2 in [c2,d2]}
	when k=0 sync start2 do {x2' =0} goto start;
	when True sync tau do {x2'=x2 , k'=k'} goto idle;
	
loc start: while x2<=a wait {dx2 in [c2,d2]}
	when True sync set2 do {k' =2,x2' =0} goto check;
	when True sync tau do {x2'=x2 , k'=k'} goto start;

loc check: while True wait {dx2 in [c2,d2]}
	when k=2 & x2>=b sync enter2 goto CS;
	when k<2 & x2>=b sync abort2 goto idle;
	when k>2 & x2>=b sync abort2 goto idle;
	when True sync tau do {x2'=x2 , k'=k'} goto check;

loc CS: while True wait {dx2 in [c2,d2]}
	when True sync release2 do {k' = 0} goto idle;
	when True sync tau do {x2'=x2 , k'=k'} goto CS;
end


automaton proc3
synclabs : tau,start3,set3,enter3,abort3,release3;

initially idle & True;

loc idle: while True wait {dx3 in [c3,d3]}
	when k=0 sync start3 do {x3' =0} goto start;
	when True sync tau do {x3'=x3 , k'=k'} goto idle;
	
loc start: while x3<=a wait {dx3 in [c3,d3]}
	when True sync set3 do {k' =3,x3' =0} goto check;
	when True sync tau do {x3'=x3 , k'=k'} goto start;

loc check: while True wait {dx3 in [c3,d3]}
	when k=3 & x3>=b sync enter3 goto CS;
	when k<3 & x3>=b sync abort3 goto idle;
	when k>3 & x3>=b sync abort3 goto idle;
	when True sync tau do {x3'=x3 , k'=k'} goto check;

loc CS: while True wait {dx3 in [c3,d3]}
	when True sync release3 do {k' = 0} goto idle;
	when True sync tau do {x3'=x3 , k'=k'} goto CS;
end

automaton proc4
synclabs : tau,start4,set4,enter4,abort4,release4;

initially idle & True;

loc idle: while True wait {dx4 in [c4,d4]}
	when k=0 sync start4 do {x4' =0} goto start;
	when True sync tau do {x4'=x4 , k'=k'} goto idle;
	
loc start: while x4<=a wait {dx4 in [c4,d4]}
	when True sync set4 do {k' =4,x4' =0} goto check;
	when True sync tau do {x4'=x4 , k'=k'} goto start;

loc check: while True wait {dx4 in [c4,d4]}
	when k=4 & x4>=b sync enter4 goto CS;
	when k<4 & x4>=b sync abort4 goto idle;
	when k>4 & x4>=b sync abort4 goto idle;
	when True sync tau do {x4'=x4 , k'=k'} goto check;

loc CS: while True wait {dx4 in [c4,d4]}
	when True sync release4 do {k' = 0} goto idle;
	when True sync tau do {x4'=x4 , k'=k'} goto CS;
end

automaton arbiter
synclabs : set1,release1,
           set2,release2,
           set3,release3,
           set4,release4,
	   tau;
	   
initially always & True;

loc always: while True wait {dk in [0,0]}
	when True sync set1 do {k'=k'} goto always;
	when True sync release1 do {k'=k'} goto always;
	when True sync set2 do {k'=k'} goto always;
	when True sync release2 do {k'=k'} goto always;
	when True sync set3 do {k'=k'} goto always;
	when True sync release3 do {k'=k'} goto always;
	when True sync set4 do {k'=k'} goto always;
	when True sync release4 do {k'=k'} goto always;
	when True sync tau do {k'=k} goto always;
end
	
--

var 
   init_reg, final_reg, reached : region;

init_reg:= loc[proc1] = idle & loc[proc2] = idle & loc[proc3] = idle & loc[proc4] = idle;

final_reg:= (loc[proc1] = CS & loc[proc2] = CS) |
            (loc[proc1] = CS & loc[proc3] = CS) |
            (loc[proc1] = CS & loc[proc4] = CS) |
            (loc[proc2] = CS & loc[proc3] = CS) |
            (loc[proc2] = CS & loc[proc4] = CS) |
            (loc[proc3] = CS & loc[proc4] = CS);

reached:= reach forward from init_reg endreach;

prints "Condition for faulty system";

print omit all locations hide non_parameters in reached & final_reg endhide;
--print reached & final_reg;



